import json
import pytest
from oam.file.file import File
from oam.asset import Asset
from oam.asset import AssetType

def test_file_key():
    want = "file:///var/html/index.html"
    f = File(url=want, name="index.html", type="Document")

    assert f.key == want

def test_file_asset_type():
    assert issubclass(File, Asset)  # Verify proper implementation of the Asset interface

    f = File(url="")
    expected = AssetType.File
    actual = f.asset_type

    assert actual == expected

def test_file_json():
    f = File(
        url="file:///var/html/index.html",
        name="index.html",
        type="Document",
    )
    expected = {
        "url": "file:///var/html/index.html",
        "name": "index.html",
        "type": "Document"
    }
    actual = f.to_dict()

    assert actual == expected



from dataclasses import dataclass
from dataclasses import field
from typing import List, Optional
from enum import Enum
from oam.asset import Asset, AssetType

@dataclass
class Organization(Asset):
    """Organization represents an organization.
    Should support relationships for the following:
    - Principal place of business
    - Parent organizations
    - Subsidiaries
    - Sister companies
    - DUNS number
    - Tax identification number
    - Trader identification number
    - ARIN organization identifier
    - Decentralized identifier (DID)
    - Ticker symbol
    - Global Location Number (GLN)
    - ISIC, NAICS, SIC, BIC, and ISO 6523 code
    - Legal Entity Identifier (LEI) ISO 17442 code
    - Registration number
    - Website
    - Social media profiles
    - Contact information
    - Founder, sponsorships, and funding sources
    """
    id: str = field(metadata={"json":"unique_id"})
    name: str
    legal_name:      Optional[str] = None
    founding_date:   Optional[str] = None
    jurisdiction:    Optional[str] = None
    registration_id: Optional[str] = None
    industry:        Optional[str] = None
    target_markets:  Optional[List[str]] = None
    active:          Optional[bool] = None
    non_profit:      Optional[bool] = None
    headcount:       Optional[int] = None

    @property
    def key(self) -> str:
        return self.id

    @property
    def asset_type(self) -> AssetType:
        return AssetType.Organization



import json
import pytest
from oam.certificate.tls_certificate import TLSCertificate, AssetType

def test_tls_certificate_key():
    want = "12345"
    cert = TLSCertificate(
        subject_common_name="www.example.org",
        issuer_common_name="DigiCert TLS RSA SHA256 2020 CA1",
        not_before="2006-01-02T15:04:05Z07:00",
        not_after="2006-01-02T15:04:05Z07:00",
        version="",
        serial_number=want,
        key_usage=[],
        ext_key_usage=[],
        signature_algorithm="",
        public_key_algorithm="",
        is_ca=False,
        crl_distribution_points=[],
        subject_key_id="",
        authority_key_id=""
    )
    
    assert cert.key == want

def test_tls_certificate_asset_type():
    cert = TLSCertificate(
        subject_common_name="www.example.org",
        issuer_common_name="DigiCert TLS RSA SHA256 2020 CA1",
        not_before="2006-01-02T15:04:05Z07:00",
        not_after="2006-01-02T15:04:05Z07:00",
        version="",
        serial_number="",
        key_usage=[],
        ext_key_usage=[],
        signature_algorithm="",
        public_key_algorithm="",
        is_ca=False,
        crl_distribution_points=[],
        subject_key_id="",
        authority_key_id=""
    )
    want = AssetType.TLSCertificate

    assert cert.asset_type == want

def test_tls_certificate_json():
    cert = TLSCertificate(
        subject_common_name="www.example.org",
        issuer_common_name="DigiCert TLS RSA SHA256 2020 CA1",
        not_before="2006-01-02T15:04:05Z07:00",
        not_after="2006-01-02T15:04:05Z07:00",
        version="",
        serial_number="",
        key_usage=[],
        ext_key_usage=[],
        signature_algorithm="",
        public_key_algorithm="",
        is_ca=False,
        crl_distribution_points=[],
        subject_key_id="",
        authority_key_id=""
    )

    # test AssetType method
    assert cert.asset_type == AssetType.TLSCertificate

    # test JSON method
    expected_json = {
        "version": "",
        "serial_number": "",
        "subject_common_name": "www.example.org",
        "issuer_common_name": "DigiCert TLS RSA SHA256 2020 CA1",
        "not_before": "2006-01-02T15:04:05Z07:00",
        "not_after": "2006-01-02T15:04:05Z07:00",
        "key_usage": [],
        "ext_key_usage": [],
        "signature_algorithm": "",
        "public_key_algorithm": "",
        "is_ca": False,
        "crl_distribution_points": [],
        "subject_key_id": "",
        "authority_key_id": ""
    }
    json_output = cert.to_dict()
    
    assert json_output == expected_json



from dataclasses import dataclass
from dataclasses import field
from oam.property import Property, PropertyType

@dataclass
class SourceProperty(Property):
    """SourceProperty represents a source of data in the graph."""
    source: str = field(metadata={"json":"name"})
    confidence: int

    @property
    def name(self) -> str:
        return self.source

    @property
    def value(self) -> str:
        return str(self.confidence)

    @property
    def property_type(self) -> PropertyType:
        return PropertyType.SourceProperty



from enum import Enum
from typing import Optional
from dataclasses import dataclass
from dataclasses import field
from oam.relation import Relation
from oam.relation import RelationType
from oam.property import Property
from oam.property import PropertyType
from oam.oam_object import OAMObject

@dataclass
class BasicDNSRelation(Relation):
    """BasicDNSRelation is a relation in the graph representing a
    basic DNS resource record."""
    name:    str = field(metadata={"json":"label"})
    rrtype:  int = field(metadata={"json":"header_rrtype"})
    cls:     Optional[int] = field(default=None, metadata={"json":"header_class"})
    ttl:     Optional[int] = field(default=None, metadata={"json":"header_ttl"})


    @property
    def relation_type(self) -> RelationType:
        return RelationType.BasicDNSRelation

    @property
    def label(self) -> str:
        return self.name

@dataclass
class PrefDNSRelation(Relation):
    """PrefDNSRelation is a relation in the graph representing a DNS
    resource record with preference information."""
    name:       str = field(metadata={"json":"label"})
    preference: int
    rrtype:     int = field(metadata={"json":"header_rrtype"})
    cls:        Optional[int] = field(default=None, metadata={"json":"header_class"})
    ttl:        Optional[int] = field(default=None, metadata={"json":"header_ttl"})

    @property
    def relation_type(self) -> RelationType:
        return RelationType.PrefDNSRelation

    @property
    def label(self) -> str:
        return self.name

@dataclass
class SRVDNSRelation(Relation):
    """SRVDNSRelation is a relation in the graph representing a DNS
    SRV resource record."""
    name:     str = field(metadata={"json":"label"})
    priority: int
    weight:   int
    port:     int
    rrtype:   int = field(metadata={"json":"header_rrtype"})
    cls:      Optional[int] = field(default=None, metadata={"json":"header_class"})
    ttl:      Optional[int] = field(default=None, metadata={"json":"header_ttl"})

    @property
    def relation_type(self) -> RelationType:
        return RelationType.SRVDNSRelation
    
    @property
    def label(self) -> str:
        return self.name

@dataclass
class DNSRecordProperty(Property):
    """DNSRecordProperty represents a DNS resource record that does
    not refer to another asset in the graph."""
    property_name: str
    data:          str
    rrtype:        int = field(metadata={"json":"header_rrtype"})
    cls:           Optional[int] = field(default=None, metadata={"json":"header_class"})
    ttl:           Optional[int] = field(default=None, metadata={"json":"header_ttl"})

    @property
    def property_type(self) -> PropertyType:
        return PropertyType.DNSRecordProperty
    
    @property
    def name(self) -> str:
        return self.property_name

    @property
    def value(self) -> str:
        return self.data



import json
import pytest
from oam.registration.autnum_record import AutnumRecord
from oam.asset import Asset
from oam.asset import AssetType

def test_autnum_record_key():
    want = "AS26808"
    as_record = AutnumRecord(
        number=26808,
        handle="AS26808",
        name="UTICA-COLLEGE",
        whois_server="whois.arin.net",
        created_date="2002-11-25 22:25:46",
        updated_date="2021-05-03 17:59:17",
        status=["active"],
    )

    assert as_record.key == want

def test_autnum_record_asset_type():
    assert issubclass(AutnumRecord, Asset)  

    w = AutnumRecord(
        number=26808,
        handle="AS26808",
        name="UTICA-COLLEGE",
        whois_server="whois.arin.net",
        created_date="2002-11-25 22:25:46",
        updated_date="2021-05-03 17:59:17",
        status=["active"],
    )
    want = AssetType.AutnumRecord

    assert w.asset_type == want

def test_autnum_record():
    record = AutnumRecord(
        number=26808,
        handle="AS26808",
        name="UTICA-COLLEGE",
        whois_server="whois.arin.net",
        created_date="2002-11-25 22:25:46",
        updated_date="2021-05-03 17:59:17",
        status=["active"],
    )

    # Test AssetType method
    assert record.asset_type == AssetType.AutnumRecord

    # Test JSON method
    expected_json = {
        "number": 26808,
        "handle": "AS26808",
        "name": "UTICA-COLLEGE",
        "whois_server": "whois.arin.net",
        "created_date": "2002-11-25 22:25:46",
        "updated_date": "2021-05-03 17:59:17",
        "status": ["active"]
    }
    json_output = record.to_dict()
    
    assert json_output == expected_json



from .asset import Asset
from .asset import AssetType
from .asset import AssetList
from .property import Property
from .property import PropertyType
from .property import PropertyList
from .relation import Relation
from .relation import RelationType
from .relation import RelationList
from .relation import asset_type_relations
from .relation import get_asset_outgoing_relations
from .relation import get_transform_asset_types
from .relation import valid_relationship
from .utils import get_asset_by_type
from .utils import get_property_by_type
from .utils import get_relation_by_type
from .utils import describe_oam_object
from .utils import make_oam_object_from_dict
from .oam_object import OAMObject

from .general import *
from .network import *
from .account import *
from .certificate import *
from .contact import *
from .dns import *
from .file import *
from .financial import *
from .org import *
from .people import *
from .pf import *
from .registration import *
from .url import *




import json
import pytest
from oam.people.person import Person
from oam.asset import AssetType
from oam.asset import Asset

def test_person_key():
    want = "222333444"
    p = Person(id=want, full_name="Jeff Foley")

    assert p.key == want


def test_person_asset_type():
    assert issubclass(Person, Asset)

    p = Person(id="", full_name="")
    assert p.asset_type == AssetType.Person


def test_person_json():
    p = Person(
        id="222333444",
        full_name="John Jacob Doe",
        first_name="John",
        middle_name="Jacob",
        family_name="Doe",
        birth_date="01/01/1970",
        gender="Male",
    )

    expected_json = {
        "unique_id": "222333444",
        "full_name": "John Jacob Doe",
        "first_name": "John",
        "middle_name": "Jacob",
        "family_name": "Doe",
        "birth_date": "01/01/1970",
        "gender": "Male"
    }

    json_data = p.to_dict()

    assert json_data == expected_json



from dataclasses import dataclass
from typing import List
from oam.asset import Asset
from oam.asset import AssetType
from enum import Enum

class TLSKeyUsageType(str, Enum):
    KeyUsageDigitalSignature = "Digital Signature"
    KeyUsageContentCommitment = "Content Commitment"
    KeyUsageKeyEncipherment = "Key Encipherment"
    KeyUsageDataEncipherment = "Data Encipherment"
    KeyUsageKeyAgreement = "Key Agreement"
    KeyUsageCertSign = "Certificate Sign"
    KeyUsageCRLSign = "CRL Sign"
    KeyUsageEncipherOnly = "Encipher Only"
    KeyUsageDecipherOnly = "Decipher Only"

class TLSExtKeyUsageType(str, Enum):
    ExtKeyUsageAny = "Any Usage"
    ExtKeyUsageServerAuth = "TLS Server Authentication"
    ExtKeyUsageClientAuth = "TLS Client Authentication"
    ExtKeyUsageCodeSigning = "Code Signing"
    ExtKeyUsageEmailProtection = "E-mail Protection"
    ExtKeyUsageIPSECEndSystem = "IPSec End System"
    ExtKeyUsageIPSECTunnel = "IPSec Tunnel"
    ExtKeyUsageIPSECUser = "IPSec User"
    ExtKeyUsageTimeStamping = "Trusted Timestamping"
    ExtKeyUsageOCSPSigning = "OCSP Signing"
    ExtKeyUsageMicrosoftServerGatedCrypto = "Microsoft Server Gated Crypto"
    ExtKeyUsageNetscapeServerGatedCrypto = "Netscape Server Gated Crypto"
    ExtKeyUsageMicrosoftCommercialCodeSigning = "Microsoft Commercial Code Signing"
    ExtKeyUsageMicrosoftKernelCodeSigning = "Microsoft Kernel Code Signing"
    
@dataclass
class TLSCertificate(Asset):
    """TLSCertificate represents a TLS Certificate asset."""
    version:                  str
    serial_number:            str
    subject_common_name:      str
    issuer_common_name:       str
    not_before:               str
    not_after:                str
    key_usage:                List[TLSKeyUsageType]
    ext_key_usage:            List[TLSExtKeyUsageType]
    signature_algorithm:      str
    public_key_algorithm:     str
    is_ca:                    bool
    crl_distribution_points:  List[str]
    subject_key_id:           str
    authority_key_id:         str

    @property
    def key(self) -> str:
        return self.serial_number

    @property
    def asset_type(self) -> AssetType:
        return AssetType.TLSCertificate



from dataclasses import dataclass
from typing import Optional
from oam.asset import Asset
from oam.asset import AssetType

@dataclass
class Location(Asset):
    """Location represents the street address location."""
    address:         str
    city:            Optional[str] = None
    building:        Optional[str] = None
    building_number: Optional[str] = None
    street_name:     Optional[str] = None
    unit:            Optional[str] = None
    po_box:          Optional[str] = None
    locality:        Optional[str] = None
    province:        Optional[str] = None
    country:         Optional[str] = None
    postal_code:     Optional[str] = None
    gln:             Optional[int] = None

    @property
    def key(self) -> str:
        return self.address

    @property
    def asset_type(self) -> AssetType:
        return AssetType.Location



from enum import Enum
from oam.asset import Asset
from oam.asset import AssetType
from dataclasses import dataclass
from dataclasses import field
from typing import Optional

class IdentifierType(str, Enum):
    AccountNumber = "account"     # Account number
    ARINHandle = "arin"           # American Registry for Internet Numbers (ARIN) handle
    ASINNumber = "asin"           # Amazon Standard Identification Number
    BankIDCode = "bic"            # Bank Identifier Code
    DUNSNumer = "duns"            # Data Universal Numbering System, Dun & Bradstreet (D&B) assigns this number
    EAN8 = "ean_8"                # International Article Number (EAN) 8-digit
    EAN13 = "ean_13"              # International Article Number (EAN) 13-digit
    EmailAddress = "email"        # Email address
    EmployerIDNumber = "ein"      # Employer Identification Number
    GlobalLocNumber = "gln"       # Global Location Number
    GlobalTradeItemNumber = "gtin" # Global Trade Item Number
    GovIssuedIDNumber = "gov_id"  # Government issued identification number for individuals
    IBANCode = "iban"             # International Bank Account Number
    ICANNAuthCode = "icann_auth"  # Auth Code, Internet Corporation for Assigned Names and Numbers
    ICANNEPPCode = "icann_epp"    # Extensible Provisioning Protocol (EPP) status code, ICANN
    ISBN = "isbn"                 # International Standard Book Number
    ISICCode = "isic"             # International Standard Industrial Classification
    LegalName = "legal_name"      # Legal name
    LEICode = "lei"               # Legal Entity Identifier, Global Legal Entity Identifier Foundation assigns this number
    MarketIDCode = "mic"          # Market Identifier Code, International Organization for Standardization (ISO) 10383 standard
    ModelNumber = "model"         # Model number
    MPN = "mpn"                   # Manufacturer Part Number
    NAICSCode = "naics"          # North American Industry Classification System
    NSNCode = "nsn"               # NATO Stock Number
    OECDCode = "oecd"             # Organization for Economic Co-operation and Development
    OpenCorpID = "ocid"           # OpenCorporates ID, identifier for a corporation in the OpenCorporates database
    OrganizationName = "org_name"  # Organization name
    SerialNumber = "serial"        # Serial number
    SICCode = "sic"                # Standard Industrial Classification
    SPGlobalCompanyID = "spglobal" # S&P Global Company ID, S&P Global Market Intelligence
    SWIFTCode = "swift"            # Society for Worldwide Interbank Financial Telecommunication
    TaxIDNumber = "tin"            # Taxpayer Identification Number
    TickerSymbol = "ticker"        # Stock ticker symbol
    UPCNumberA = "upc_a"           # Universal Product Code (UPC) Version A
    UPCNumberE = "upc_e"           # Universal Product Code (UPC) Version E
    VersionNumber = "version"      # Version number

@dataclass
class Identifier(Asset):
    """Identifier identifies something that's a member of a system or
    organization that issues ID numbers or codes.
    
    Should support relationships for the following:
    - Registration agency (e.g., ContactRecord)
    - Issuing authority (e.g., ContactRecord)
    - Issuing agent (e.g., ContactRecord)
    """
    unique_id:       str
    id:              str
    type:            IdentifierType = field(metadata={"json":"id_type"})
    creation_date:   Optional[str] = None
    update_date:     Optional[str] = None
    expiration_date: Optional[str] = None
    status:          Optional[str] = None

    @property
    def key(self) -> str:
        return self.unique_id

    @property
    def asset_type(self) -> AssetType:
        return AssetType.Identifier



import json
import pytest
from oam.pf.product import Product
from oam.pf.product import ProductRelease
from oam.asset import Asset
from oam.asset import AssetType

def test_product_key():
    want = "12345"
    p = Product(id=want, name="OWASP Amass", type="Information Security")

    assert p.key == want

def test_product_asset_type():
    assert issubclass(Product, Asset)
    assert issubclass(Product, Asset)
    p = Product(id="dummy", name="dummy", type="dummy")

    assert p.asset_type == AssetType.Product

def test_product_json():
    p = Product(
        id="12345",
        name="OWASP Amass",
        type="Attack Surface Management",
        category="Information Security",
        description="In-depth attack surface mapping and asset discovery",
        country_of_origin="US"
    )
    expected = {
        "unique_id": "12345",
        "product_name": "OWASP Amass",
        "product_type": "Attack Surface Management",
        "category": "Information Security",
        "description": "In-depth attack surface mapping and asset discovery",
        "country_of_origin": "US"
    }
    
    actual = p.to_dict()
    
    assert actual == expected

def test_product_release_key():
    want = "Amass v4.2.0"
    p = ProductRelease(name=want)

    assert p.key == want


def test_product_release_asset_type():
    assert issubclass(ProductRelease, Asset)
    assert issubclass(ProductRelease, Asset)
    p = ProductRelease(name="dummy")

    assert p.asset_type == AssetType.ProductRelease

def test_product_release_json():
    p = ProductRelease(
        name="Amass v4.2.0",
        release_date="2023-09-10T14:15:00Z"
    )
    expected = {
        "name": "Amass v4.2.0",
        "release_date": "2023-09-10T14:15:00Z"
    }
    
    actual = p.to_dict()
    
    assert actual == expected



from dataclasses import dataclass
from typing import Optional
from oam.property import Property, PropertyType


@dataclass
class VulnProperty(Property):
    """VulnProperty represents a simple property in the graph."""
    id:          str
    description: str
    source:      Optional[str] = None
    category:    Optional[str] = None
    enumeration: Optional[str] = None
    reference:   Optional[str] = None

    @property
    def name(self) -> str:
        return self.id

    @property
    def value(self) -> str:
        return self.description

    @property
    def property_type(self) -> PropertyType:
        return PropertyType.VulnProperty

    def to_dict(self) -> dict:
        return {key: value for key, value in {
            "id":       self.id,
            "desc":     self.description,
            "source":   self.source,
            "category": self.category,
            "enum":     self.enumeration,
            "ref":      self.reference,
        }.items() if value is not None}



import pytest
import json
from oam.general.port import PortRelation
from oam.relation import Relation
from oam.relation import RelationType

def test_port_relation_name():
    want = "port"
    pr = PortRelation(
        name=want,
        port_number=80,
        protocol="tcp"
    )

    assert pr.label == want

def test_port_relation_implements_relation():
    assert issubclass(PortRelation, Relation)

def test_port_relation():
    pr = PortRelation(
        name="port",
        port_number=80,
        protocol="tcp"
    )

    assert pr.name == "port"
    assert pr.port_number == 80
    assert pr.protocol == "tcp"
    assert pr.relation_type == RelationType.PortRelation

def test_port_relation_json_serialization():
    pr = PortRelation(
        name="port",
        port_number=80,
        protocol="tcp"
    )

    json_data = pr.to_dict()
    expected_json = {
        "label": "port",
        "port_number": 80,
        "protocol": "tcp"
    }
    
    assert json_data == expected_json



import json
import pytest
from oam.contact.phone import Phone
from oam.contact.phone import PhoneType
from oam.asset import Asset
from oam.asset import AssetType

def test_phone_key():
    want = "+12345556666"
    p = Phone.from_text(want)
    assert p.key == want

def test_phone_asset_type():
    p = Phone.from_text("+12345556666")
    assert isinstance(p, Asset)
    assert p.asset_type == AssetType.Phone

def test_phone_json():
    p = Phone(
        raw="123-456-7890 Ext. 123",
        e164="+1234567890",
        type=PhoneType.Mobile,
        country_abbrev="US",
        country_code=1,
        ext="123",
    )

    expected_json = {
        "raw":"123-456-7890 Ext. 123",
        "e164":"+1234567890",
        "type":"mobile",
        "country_abbrev":"US",
        "country_code":1,
        "ext":"123"
    }

    phone_json = p.to_dict()
    assert phone_json == expected_json



import pytest
import json
from oam.contact.contact_record import ContactRecord
from oam.asset import Asset
from oam.asset import AssetType

def test_contact_record_key():
    want = "https://owasp.org/contacts"
    cr = ContactRecord(discovered_at=want)
    
    assert cr.key == want

def test_contact_record_asset_type():
    cr = ContactRecord(discovered_at="https://owasp.org/contacts")
    
    assert issubclass(ContactRecord, Asset)
    assert cr.asset_type == AssetType.ContactRecord

def test_contact_record_json():
    cr = ContactRecord(discovered_at="https://owasp.org")
    
    expected_json = {
        "discovered_at": "https://owasp.org"
    }
    
    json_data = cr.to_dict()
    
    assert json_data == expected_json



import json
from json import JSONDecodeError
from oam.dns.fqdn import FQDN
from oam.asset import Asset
from oam.asset import AssetType

def test_fqdn_key():
    want = "example.com"
    fqdn = FQDN(name=want)

    assert fqdn.key == want

def test_fqdn_implements_asset():
    assert issubclass(FQDN, Asset)

def test_successful_creation_of_fqdn_with_valid_name_and_tld():
    fqdn = FQDN(name="foo.example.com")
    
    assert fqdn.name == "foo.example.com"
    assert fqdn.asset_type == AssetType.FQDN

def test_successful_json_serialization_of_fqdn_with_valid_name_and_tld():
    fqdn = FQDN(name="foo.example.com")
    
    json_data = fqdn.to_dict()
    expected_json = {"name":"foo.example.com"}
    assert json_data == expected_json





from dataclasses import dataclass
from oam.asset import Asset
from oam.asset import  AssetType
from enum import Enum

class IPAddressType(str, Enum):
    IPv4 = "IPv4"
    IPv6 = "IPv6"

@dataclass
class IPAddress(Asset):
    """IPAddress represents an IP address."""
    address: str
    type:    IPAddressType

    @property
    def key(self) -> str:
        return self.address

    @property
    def asset_type(self) -> AssetType:
        return AssetType.IPAddress



from dataclasses import dataclass
from typing import List, Optional
from oam.asset import Asset, AssetType
import json


@dataclass
class DomainRecord(Asset):
    """DomainRecord represents the WHOIS record of a domain."""
    domain:          str
    id:              Optional[str] = None
    raw:             Optional[str] = None
    punycode:        Optional[str] = None
    name:            Optional[str] = None
    extension:       Optional[str] = None
    whois_server:    Optional[str] = None
    created_date:    Optional[str] = None
    updated_date:    Optional[str] = None
    expiration_date: Optional[str] = None
    status:          Optional[List[str]] = None
    dnssec:          Optional[bool] = None

    @property
    def key(self) -> str:
        return self.domain

    @property
    def asset_type(self) -> AssetType:
        return AssetType.DomainRecord



import pytest
import json
from oam.org.org import Organization
from oam.asset import AssetType

def test_organization_key():
    want = "222333444"
    o = Organization(id=want, name="OWASP Foundation")
    assert o.key == want

def test_organization_asset_type():
    o = Organization(id="222333444", name="OWASP Foundation")
    expected = AssetType.Organization
    actual = o.asset_type
    assert actual == expected

def test_organization_json():
    o = Organization(
        id="222333444",
        name="Acme Inc.",
        legal_name="Acme Inc.",
        founding_date="2013-07-24T14:15:00Z",
        jurisdiction="US-DE",
        registration_id="123456789",
        industry="Technology",
        target_markets=["Apps", "Software"],
        active=True,
        non_profit=False,
        headcount=10000,
    )
    expected = {
        "unique_id": "222333444",
        "name": "Acme Inc.",
        "legal_name": "Acme Inc.",
        "founding_date": "2013-07-24T14:15:00Z",
        "jurisdiction": "US-DE",
        "registration_id": "123456789",
        "industry": "Technology",
        "target_markets": ["Apps", "Software"],
        "active": True,
        "non_profit": False,
        "headcount": 10000
    }
    actual = o.to_dict()
    assert actual == expected



from dataclasses import dataclass
from typing import Optional
from oam.asset import Asset, AssetType

@dataclass
class File(Asset):
    """File represents a file discovered, such as a document or image."""
    url: str
    name: Optional[str] = None
    type: Optional[str] = None

    @property
    def key(self) -> str:
        return self.url

    @property
    def asset_type(self) -> AssetType:
        return AssetType.File



import pytest
from oam.general.source import SourceProperty
from oam.property import Property
from oam.property import PropertyType

def test_source_property_name():
    want = "anything"
    sp = SourceProperty(source="anything", confidence=80)
    
    assert sp.name == want

def test_source_property_value():
    want = "80"
    sp = SourceProperty(source="anything", confidence=80)
    
    assert sp.value == want

def test_source_property_implements_property():
    assert issubclass(SourceProperty, Property)

def test_source_property_creation():
    sp = SourceProperty(source="anything", confidence=80)

    assert sp.source == "anything"
    assert sp.confidence == 80
    assert sp.property_type == PropertyType.SourceProperty

def test_source_property_json_serialization():
    sp = SourceProperty(source="anything", confidence=80)

    expected = {
        "name": "anything",
        "confidence": 80
    }
    
    assert sp.to_dict() == expected



import json
import pytest
from oam.network.autonomous_system import AutonomousSystem
from oam.asset import Asset
from oam.asset import AssetType

def test_autonomous_system_key():
    want = 26808
    as_system = AutonomousSystem(number=want)

    assert as_system.key == str(want)

def test_autonomous_system_asset():
    assert issubclass(AutonomousSystem, Asset)

def test_autonomous_system_creation():
    as_system = AutonomousSystem(number=64496)

    assert as_system.number == 64496
    assert as_system.asset_type == AssetType.AutonomousSystem

def test_autonomous_system_json():
    as_system = AutonomousSystem(number=64496)

    expected_json = {"number": 64496}
    assert as_system.to_dict() == expected_json



import pytest
import json
from oam.financial.funds_transfer import FundsTransfer
from oam.asset import AssetType

def test_funds_transfer_key():
    want = "222333444"
    ft = FundsTransfer(id=want, amount=42.0)

    assert ft.key == want

def test_funds_transfer_asset_type():
    ft = FundsTransfer(id="222333444", amount=42.0)
    expected = AssetType.FundsTransfer
    actual = ft.asset_type

    assert actual == expected

def test_funds_transfer_json():
    ft = FundsTransfer(
        id="222333444",
        amount=42.0,
        reference_number="555666777",
        currency="US",
        method="ACH",
        exchange_date="2013-07-24T14:15:00Z",
        exchange_rate=0.9,
    )
    expected = {
        "unique_id": "222333444",
        "amount": 42.0,
        "reference_number": "555666777",
        "currency": "US",
        "transfer_method": "ACH",
        "exchange_date": "2013-07-24T14:15:00Z",
        "exchange_rate": 0.9
    }
    
    assert ft.to_dict() == expected



from dataclasses import dataclass
from oam.asset import AssetType
from oam.asset import Asset

@dataclass
class FQDN(Asset):
    """FQDN represents a Fully Qualified Domain Name."""
    name: str
    
    @property
    def key(self) -> str:
        return self.name

    @property
    def asset_type(self) -> AssetType:
        return AssetType.FQDN



import json
from oam.oam_object import OAMObject
from abc import ABC
from abc import abstractmethod
from enum import Enum

class AssetType(str, Enum):
    Account = "Account"
    AutnumRecord = "AutnumRecord"
    AutonomousSystem = "AutonomousSystem"
    ContactRecord = "ContactRecord"
    DomainRecord = "DomainRecord"
    File = "File"
    FQDN = "FQDN"
    FundsTransfer = "FundsTransfer"
    Identifier = "Identifier"
    IPAddress = "IPAddress"
    IPNetRecord = "IPNetRecord"
    Location = "Location"
    Netblock = "Netblock"
    Organization = "Organization"
    Person = "Person"
    Phone = "Phone"
    Product = "Product"
    ProductRelease = "ProductRelease"
    Service = "Service"
    TLSCertificate = "TLSCertificate"
    URL = "URL"

class Asset(OAMObject):
    @property
    @abstractmethod
    def key(self) -> str:
        pass

    @property
    @abstractmethod
    def asset_type(self) -> AssetType:
        pass

AssetList = list(AssetType)




from __future__ import annotations
from abc import ABC
from json import dumps
from dataclasses import fields
from dataclasses import dataclass
from dataclasses import is_dataclass
from typing import Any

@dataclass
class OAMObject(ABC):
    def to_dict(self) -> dict:
        d = {}
        for field in fields(self):
            json_name = field.metadata["json"] if "json" in field.metadata else field.name
            json_value = self.__dict__[field.name]
            if json_value is not None:
                d[json_name] = json_value
        return d
    
    def to_json(self) -> str:
        return dumps(self.to_dict())        



from dataclasses import dataclass
from dataclasses import field
from enum import Enum
from typing import Optional
from oam.asset import Asset, AssetType

@dataclass
class Person(Asset):
    """Person represents a person's information."""
    id:          str = field(metadata={"json":"unique_id"})
    full_name:   str
    first_name:  Optional[str] = None
    family_name: Optional[str] = None
    middle_name: Optional[str] = None
    birth_date:  Optional[str] = None
    gender:      Optional[str] = None

    @property
    def key(self) -> str:
        return self.id

    @property
    def asset_type(self) -> AssetType:
        return AssetType.Person



from dataclasses import dataclass
from enum import Enum
from typing import Optional
from oam.asset import Asset
from oam.asset import AssetType
from phonenumbers import parse
from phonenumbers import format_number
from phonenumbers import PhoneNumberFormat
from phonenumbers.phonenumberutil import region_code_for_country_code

class PhoneType(str, Enum):
    Regular = "phone"
    Fax     = "fax"
    Mobile  = "mobile"

@dataclass
class Phone(Asset):
    """This struct represents the phone number, whether it is fax,
    mobile, or home number linked to the possible asset."""
    raw:            str
    e164:           Optional[str] = None  # E.164 format
    type:           Optional[str] = None
    country_abbrev: Optional[str] = None
    country_code:   Optional[int] = None
    ext:            Optional[str] = None

    @property
    def key(self) -> str:
        return self.raw

    @property
    def asset_type(self) -> AssetType:
        return AssetType.Phone
    
    @staticmethod
    def from_text(phone: str, region: Optional[str] = None) -> 'Phone':
        o = parse(phone, region)
        e164 = format_number(o, PhoneNumberFormat.E164)
        if not region and o.country_code:
            region = region_code_for_country_code(o.country_code)
            
        return Phone(
            raw = phone,
            e164 = e164,
            country_code = o.country_code,
            country_abbrev = region,
            ext = o.extension or None
        )



from dataclasses import dataclass
from oam.asset import Asset
from oam.asset import AssetType
from enum import Enum

class NetblockType(str, Enum):
    IPv4 = "IPv4"
    IPv6 = "IPv6"

@dataclass
class Netblock(Asset):
    """Netblock represents a block of IP addresses in a network."""
    cidr: str
    type: NetblockType

    @property
    def key(self) -> str:
        return self.cidr

    @property
    def asset_type(self) -> AssetType:
        return AssetType.Netblock
